<?php

namespace InSegment\WorkersFramework\Console\Commands;

use Shavshukov\EventAppBase\Console\AppCommand;

use InSegment\WorkersFramework\Providers\QueueConnectionServiceProvider;
use InSegment\WorkersFramework\Providers\LockfileServiceProvider;
use InSegment\WorkersFramework\Interfaces\ConnectionListInterface;
use InSegment\WorkersFramework\Services\Manager;
use InSegment\WorkersFramework\Models\WorkerPidKiller;

class Manage extends AppCommand
{
    
    /**
     * {@inheritDoc}
     */
    public function usage(): string
    {
        return 'Manage the queue service: run workers and a web server';
    }

    /**
     * {@inheritDoc}
     */
    public function handle(array $arguments)
    {
        (new WorkerPidKiller)->killPreviousPids($this->app);
        $this->app->registerProviders([
            new QueueConnectionServiceProvider($this->app),
            new LockfileServiceProvider($this->app),
        ], $this->app);

        return $this->app->service('queue-connections-list')->then(function (ConnectionListInterface $queueDriver) {
            return (new Manager($this->app, $queueDriver))->manage();
        });
    }

    /**
     * {@inheritDoc}
     */
    public function help(string $entryFileName): string
    {
        return implode("\n", [
            "Usage: {$entryFileName} manage\n",
            'Manage the queue service: run workers and a web server'
        ]);
    }

}
