<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use InSegment\WorkersFramework\Console\Application;

class JustPublishMessage
{
    /**
     * Application instance
     *
     * @var \InSegment\WorkersFramework\Console\Application 
     */
    protected $app;
    
    /**
     * Connections config
     *
     * @var \InSegment\WorkersFramework\Queues\ConnectionsConfig 
     */
    protected $connectionsConfig;
    
    /**
     * Constructor
     * 
     * @param string $appDir
     * @param string $configName
     */
    public function __construct(string $appDir, string $configName)
    {
        $this->app = new Application($appDir, $configName);
        $this->connectionsConfig = $this->app->config()->getConnectionsConfig();
    }
    
    /**
     * Just publish a message to a queue of one of the connections
     * 
     * @param string $connectionName
     * @param string $queue
     * @param string $message
     * @param array $headers
     * @return mixed
     * @throws \Exception
     */
    public function publishMessage(string $connectionName, string $queue, string $message, array $headers = [])
    {
        $connectors = $this->connectionsConfig->getConnectors();
        
        if (!isset($connectors[$connectionName])) {
            throw new \Exception("Connector for '{$connectionName}' does not exists");
        }
        
        $connector = $connectors[$connectionName];
        
        if (!$connector instanceof Connector) {
            throw new \Exception("Connector for '{$connectionName}' is not for rabbit driver");
        }
        
        $channel = $connector->createSyncDriver();
        
        return $channel->publish(
            $message, // String
            $headers, // Headers
            '', // Exchange name
            $queue // Routing key
        );
    }
    
}
