<?php

namespace InSegment\WorkersFramework\Queues;

use InSegment\WorkersFramework\Interfaces\QueueInterface;

class ServiceMessenger
{
    /**
     * Instance of queue connected to the channel
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueInterface
     */
    protected $queue;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueInterface $queue
     * @param string $queueName
     */
    public function __construct(QueueInterface $queue)
    {
        $this->queue = $queue;
    }
    
    /**
     * Send updates to the service queue
     * 
     * @param array $updates
     * @param bool $updateTimestamp
     * @return \React\Promise\PromiseInterface
     */
    public function sendUpdates(array $updates, bool $updateTimestamp = true)
    {
        if ($updateTimestamp) {
            $updates['timestamp'] = microtime(true);
        }
        
        return (new UnpublishedMessage($this->queue, json_encode($updates)))->send();
    }
    
}
