<?php

namespace InSegment\WorkersFramework\Queues;

use React\Promise\PromiseInterface;
use Shavshukov\EventAppBase\Model\ArrayModel;
use Shavshukov\EventAppBase\Interfaces\ModelInterface;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;

class UnpublishedMessage implements QueueMessageInterface
{
    /**
     * Queue to publish message to
     *
     * @var \InSegment\WorkersFramework\Queues\Queue
     */
    protected $queue;
    
    /**
     * Payload of the message
     *
     * @var string
     */
    protected $contents;
    
    /**
     * Headers to publish message with
     *
     * @var \Shavshukov\EventAppBase\Interfaces\ModelInterface
     */
    protected $headers;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Queues\Queue $queue
     * @param string $contents
     * @param \Shavshukov\EventAppBase\Interfaces\ModelInterface|null $headers
     */
    public function __construct(Queue $queue, string $contents, ModelInterface $headers = null)
    {
        $this->queue = $queue;
        $this->contents = $contents;
        $this->headers = $headers ?? new ArrayModel();
    }
    
    /**
     * {@inheritDoc}
     */
    public function getConsumerTag(): ?string
    {
        return null;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getContents(): string
    {
        return $this->contents;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getHeaders(): ModelInterface
    {
        return $this->headers;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueueName(): string
    {
        return $this->queue->getQueueName();
    }
    
    /**
     * {@inheritDoc}
     */
    public function acknowlege()
    {
        throw new \Exception('The message was not published - it cannot be acknowledged');
    }
    
    /**
     * {@inheritDoc}
     */
    public function fail()
    {
        throw new \Exception('The message was not published - it cannot be failed');
    }
    
    /**
     * Send this message to the queue
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function send(): PromiseInterface
    {
        return $this->queue->getConnection()->publish($this);
    }
    
}
