<?php

namespace InSegment\WorkersFramework\Queues;

use InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface;
use React\Promise;

class WorkerDeclareStructure
{
    /**
     * @var \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    protected $work;
    
    /**
     * @var \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    protected $service;
    
    /**
     * @var \InSegment\WorkersFramework\Interfaces\DeclareQueueBuilderInterface
     */
    protected $dlx;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Queues\DeclareQueueBuilderInterface $work
     * @param \InSegment\WorkersFramework\Queues\DeclareQueueBuilderInterface $service
     * @param \InSegment\WorkersFramework\Queues\DeclareQueueBuilderInterface $dlx
     */
    public function __construct(DeclareQueueBuilderInterface $work, DeclareQueueBuilderInterface $service, DeclareQueueBuilderInterface $dlx = null)
    {
        $this->work = $work;
        $this->service = $service;
        $this->dlx = $dlx;
    }
    
    /**
     * Declare all the set structures
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function declareAll(): Promise\PromiseInterface
    {
        return Promise\all(array_map(function (DeclareQueueBuilderInterface $builder) {
            return $builder->doDeclare(true);
        }, array_filter([
            $this->work, $this->service, $this->dlx,
        ])));
    }
    
    /**
     * Declare only work queue and get count of messages
     * 
     * @return \React\Promise\PromiseInterface<int>
     */
    public function declareForCount(): Promise\PromiseInterface
    {
        return $this->work->doDeclare();
    }
    
}
